@@ -19,6 +19,7 @@ from time import sleep
 
 from mailchimp3 import MailChimp
 import firefly as ff
+from mailchimp3.mailchimpclient import MailChimpError
 
 from .mailchimp_client_factory import MailchimpClientFactory
 from ... import domain as domain
@@ -45,7 +46,12 @@ class MailchimpEmailService(domain.EmailService):
         except (TypeError, AttributeError):
             pass
 
-        member = client.lists.members.create(audience.meta['mc_id'], payload)
+        try:
+            member = client.lists.members.create(audience.meta['mc_id'], payload)
+        except MailChimpError as e:
+            if 'is already a list member' not in str(e):
+                raise e
+            member = client.lists.members.get(audience.meta['mc_id'], contact.email)
         audience.get_member_by_contact(contact).meta['mc_id'] = member['id']
 
     def add_tag_to_audience_member(self, tag: str, audience: domain.Audience, contact: domain.Contact):
@@ -68,32 +74,44 @@ class MailchimpEmailService(domain.EmailService):
         merge_fields = self._get_mc_merge_fields(client, audience)
 
         if create is True:
-            names = list(merge_fields.keys())
             for k, v in meta.items():
+                names = list(merge_fields.keys())
                 if k not in names:
                     attempts = 0
                     while True:
                         try:
                             attempts += 1
                             with self._mutex(f'mailchimp-tags-{audience.id}', timeout=0):
-                                merge_fields[k] = self._create_merge_field(client, audience, k, v)
-                                break
+                                new_merge_fields = self._get_mc_merge_fields(client, audience)
+                                names = list(new_merge_fields.keys())
+                                if k not in names:
+                                    merge_fields[k] = self._create_merge_field(client, audience, k, v)
+                                else:
+                                    merge_fields[k] = new_merge_fields[k]
+                            break
                         except TimeoutError:
                             if attempts >= 5:
                                 break
                             sleep(2)
-                            merge_fields = self._get_mc_merge_fields(client, audience)
-                            names = list(merge_fields.keys())
-                            if k in names:
-                                break
 
         return {v: meta[k] for k, v in merge_fields.items() if (k in meta and meta[k] is not None)}
 
     @staticmethod
     def _get_mc_merge_fields(client: MailChimp, audience: domain.Audience):
-        return {
-            x['name']: x['tag'] for x in client.lists.merge_fields.all(audience.meta['mc_id'])['merge_fields']
-        }
+        count = 25
+        offset = 0
+        ret = {}
+        while True:
+            merge_fields = client.lists.merge_fields.all(
+                audience.meta['mc_id'], count=count, offset=offset
+            )['merge_fields']
+            for x in merge_fields:
+                ret[x['name']] = x['tag']
+            if len(merge_fields) < count:
+                break
+            offset += count
+
+        return ret
 
     @staticmethod
     def _create_merge_field(client: MailChimp, audience: domain.Audience, name: str, hint: any):
